<?php
/**
 * Part of SimpleViewer.net portfolio web site package.
 *
 * @package Showkase
 * @author Jack Hardie {@link http://www.jhardie.com}
 * @copyright Copyright (c) 2014, SimpleViewer Inc.
 */
defined('SK_ACCESS')||die('<h1>403: Forbidden</h1>');
require_once 'classes'.DIRECTORY_SEPARATOR.'helpers.php';
 
/**
 * Resize images
 *
 * @package Showkase
 */
Class ImageSizer
{
    /**
     * @var integer max output width
     */
    private $maxWidth = 1024;
    
    /**
     * @var integer max output height
     */
    private $maxHeight = 768;
    
    /**
     * @var integer jpg quality
     */
    private $quality = 80;
    
    /**
     * @var integer max image size
     */
    private $maxImageBytes;
    
    /**
     * constructor
     */
    public function __construct($maxWidth, $maxHeight, $quality)
    {
        if ($maxWidth > 0) {
            $this->maxWidth = $maxWidth;
        }
        if ($maxHeight > 0) {
            $this->maxHeight = $maxHeight;
        }
        if ($quality > 0 && $quality <= 100) {
            $this->quality = $quality;
        }
        $memoryLimit = (ini_get('memory_limit') == '')
            ? MEMORY_LIMIT_FALLBACK
            : ini_get('memory_limit');
        $this->maxImageBytes = (MEMORY_LIMIT == 0)
            ? Helpers::getBytes($memoryLimit)
            : MEMORY_LIMIT * pow(2,20);
    }

    /**
     * Create a new image from a source image
     *
     * @access private
     * @return void
     * @param string path to source image
     * @param string path to destination image
     */
    function saveImage($sourcePath, $destPath)
    {
        if (DEMO_MODE) return;
        $fileName = basename($sourcePath);
        $dimensions = @getimagesize($sourcePath);
        if ($dimensions === false) {
            throw new Exception('Cannot calculate size of source image '.$sourcePath);
        }
        if ($dimensions[0] == 0 || $dimensions[1] == 0) {
            throw new Exception('Zero width or height for '.$sourcePath);
        }
        // $imageInfo['channels'] may not be set for png images so just guess at 3
        $memoryNeeded = round(($dimensions[0] * $dimensions[1] * $dimensions['bits'] * 3 / 8 + Pow(2, 16)) * MEMORY_SAFETY_FACTOR);
        if (
            $tooLarge = 
                $memoryNeeded > $this->maxImageBytes
                && $this->maxImageBytes !== -1
        ) {
        	  Board::addMessage('Image '.$fileName.' is too large to resize in available PHP memory');
        } 
    	  $imageWidth		= $dimensions[0];
    	  $imageHeight	= $dimensions[1];
        if (
            $imageWidth <= $this->maxWidth
            && $imageHeight <= $this->maxHeight
            || $tooLarge
        ) {
          if ($sourcePath == $destPath) {
              return;
          }
          if (!filer::skCopy($sourcePath, $destPath)) {
              throw new Exception('Cannot copy image to '.$destPath);
          }
          return;
        }
        $shrinkage = min(
            $this->maxWidth/$imageWidth,
            $this->maxHeight/$imageHeight
        );
        $newWidth = round($imageWidth * $shrinkage);
        $newHeight = round($imageHeight * $shrinkage);
        // Create an empty destination image. 
        $imageDest = @imagecreatetruecolor($newWidth, $newHeight);
        if ($imageDest === false) {
            @imagedestroy($imageSrc);
            throw new Exception('Cannot create true color image');
        }
        try {
            switch ($dimensions[2]) {
                case IMAGETYPE_GIF :
                    if(!$imageSrc = @imagecreatefromgif($sourcePath)) {
                        throw new Exception('Cannot create image from gif '.$fileName);
                    }
                    $grey = imagecolorallocate($imageDest, 192, 192, 192);
                    imagefill($imageDest, 0, 0, $grey);
                    if (!@imagecopyresampled($imageDest, $imageSrc, 0, 0, 0, 0, $newWidth, $newHeight, $imageWidth, $imageHeight)) {
                        throw new Exception('Cannot resample gif image '.$fileName);
                    }
                    if (!@imagegif($imageDest, $destPath)) {
                        throw new Exception('Cannot save to gif format '.$fileName);
                    }
                    break;
                case IMAGETYPE_JPEG :
                    if (!$imageSrc = @imagecreatefromjpeg($sourcePath)) {
                        throw new Exception('Cannot create image from jpg '.$fileName);
                    }
                    if (!@imagecopyresampled($imageDest, $imageSrc, 0, 0, 0, 0, $newWidth, $newHeight, $imageWidth, $imageHeight)) {
                        throw new Exception('Cannot resample jpg image '.$fileName);
                    }
                    if (!@imagejpeg($imageDest, $destPath, $this->quality)) {
                        throw new Exception('Cannot save to jpg format '.$fileName);
                    }
                    break;
                case IMAGETYPE_PNG :
                    if (!$imageSrc = @imagecreatefrompng($sourcePath)) {
                        throw new Exception('Cannot create image from png '.$fileName);
                    }
                    if (!@imagecopyresampled($imageDest, $imageSrc, 0, 0, 0, 0, $newWidth, $newHeight, $imageWidth, $imageHeight)) {
                        throw new Exception('Cannot resample png image '.$fileName);
                    }
                    if (!@imagepng($imageDest, $destPath)) {
                        throw new Exception('Cannot save to png format '.$fileName);
                    }
                    break;
                default :
                    throw new Exception('Unidentified image type '.$fileName);
            }
        } catch (Exception $e) {
            throw new Exception('Unable to resize image, '.$e->getMessage());
        }
		// Delete both image resources.
		    if (is_resource($imageSrc)) {
		        @imagedestroy($imageSrc);
		    }
		    if (is_resource($imageDest)) {
  		      @imagedestroy($imageDest);
		    }
        unset ($imageSrc, $imageDest);
    }

}
